@@ -15,20 +15,18 @@
 import os
 import shutil
 import logging
-import nbformat
 
 from tabulate import tabulate
 
-from kale.core import Kale
-from kale.nbparser import parser
-from kale.static_analysis import ast
-from kale.marshal import resource_load
+from kale import marshal
 from kale.rpc.log import create_adapter
+from kale import Compiler, NotebookProcessor
 from kale.rpc.errors import RPCInternalError
-from kale.common import podutils, kfputils, kfutils
+from kale.common import podutils, kfputils, kfutils, astutils
 
 KALE_MARSHAL_DIR_POSTFIX = ".kale.marshal.dir"
 KALE_PIPELINE_STEP_ENV = "KALE_PIPELINE_STEP"
+KALE_SNAPSHOT_FINAL_ENV = "KALE_SNAPSHOT_FINAL"
 
 logger = create_adapter(logging.getLogger(__name__))
 
@@ -78,33 +76,41 @@ def list_volumes(request):
     return volumes_out
 
 
+def get_volume_containing_path(request, path):
+    """Get the closest volume mount point to the input absolute path."""
+    vol = podutils.get_volume_containing_path(path)
+    return {"name": vol[1].name, "mount_point": vol[0]}
+
+
 def get_base_image(request):
     """Get the current pod's docker base image."""
     return podutils.get_docker_base_image()
 
 
+# fixme: Remove the debug argument from the labextension RPC call.
 def compile_notebook(request, source_notebook_path,
                      notebook_metadata_overrides=None, debug=False):
     """Compile the notebook to KFP DSL."""
-    instance = Kale(source_notebook_path, notebook_metadata_overrides, debug)
-    instance.logger = request.log if hasattr(request, "log") else logger
+    processor = NotebookProcessor(source_notebook_path,
+                                  notebook_metadata_overrides)
+    pipeline = processor.to_pipeline()
+    script_path = Compiler(pipeline).compile()
+    # FIXME: Why were we tapping into the Kale logger?
+    # instance = Kale(source_notebook_path, notebook_metadata_overrides, debug)
+    # instance.logger = request.log if hasattr(request, "log") else logger
 
-    pipeline_graph, pipeline_parameters = instance.notebook_to_graph()
-    script_path = instance.generate_kfp_executable(pipeline_graph,
-                                                   pipeline_parameters)
-
-    pipeline_name = instance.pipeline_metadata["pipeline_name"]
-    package_path = kfputils.compile_pipeline(script_path, pipeline_name)
+    package_path = kfputils.compile_pipeline(script_path,
+                                             pipeline.config.pipeline_name)
 
     return {"pipeline_package_path": os.path.relpath(package_path),
-            "pipeline_metadata": instance.pipeline_metadata}
+            "pipeline_metadata": pipeline.config.to_dict()}
 
 
 def validate_notebook(request, source_notebook_path,
                       notebook_metadata_overrides=None):
     """Validate notebook metadata."""
     # Notebook metadata is validated at class instantiation
-    Kale(source_notebook_path, notebook_metadata_overrides)
+    NotebookProcessor(source_notebook_path, notebook_metadata_overrides)
     return True
 
 
@@ -113,16 +119,15 @@ def get_pipeline_parameters(request, source_notebook_path):
     # read notebook
     log = request.log if hasattr(request, "log") else logger
     try:
-        source_notebook_path = os.path.expanduser(source_notebook_path)
-        notebook = nbformat.read(source_notebook_path,
-                                 as_version=nbformat.NO_CONVERT)
-        params_source = parser.get_pipeline_parameters_source(notebook)
+        processor = NotebookProcessor(os.path.expanduser(source_notebook_path),
+                                      skip_validation=True)
+        params_source = processor.get_pipeline_parameters_source()
         if params_source == '':
             raise ValueError("No pipeline parameters found. Please tag a cell"
                              " of the notebook with the `pipeline-parameters`"
                              " tag.")
         # get a dict from the 'pipeline parameters' cell source code
-        params_dict = ast.parse_assignments_expressions(params_source)
+        params_dict = astutils.parse_assignments_expressions(params_source)
     except ValueError as e:
         log.exception("Value Error during parsing of pipeline parameters")
         raise RPCInternalError(details=str(e), trans_id=request.trans_id)
@@ -139,16 +144,15 @@ def get_pipeline_metrics(request, source_notebook_path):
     # read notebook
     log = request.log if hasattr(request, "log") else logger
     try:
-        source_notebook_path = os.path.expanduser(source_notebook_path)
-        notebook = nbformat.read(source_notebook_path,
-                                 as_version=nbformat.NO_CONVERT)
-        metrics_source = parser.get_pipeline_metrics_source(notebook)
+        processor = NotebookProcessor(os.path.expanduser(source_notebook_path),
+                                      skip_validation=True)
+        metrics_source = processor.get_pipeline_metrics_source()
         if metrics_source == '':
             raise ValueError("No pipeline metrics found. Please tag a cell"
                              " of the notebook with the `pipeline-metrics`"
                              " tag.")
         # get a dict from the 'pipeline parameters' cell source code
-        metrics = ast.parse_metrics_print_statements(metrics_source)
+        metrics = astutils.parse_metrics_print_statements(metrics_source)
     except ValueError as e:
         log.exception("Failed to parse pipeline metrics")
         raise RPCInternalError(details=str(e), trans_id=request.trans_id)
@@ -171,12 +175,10 @@ def unmarshal_data(source_notebook_path):
     if not os.path.exists(kale_marshal_dir):
         return {}
 
-    load_file_names = [f for f in os.listdir(kale_marshal_dir)
-                       if os.path.isfile(os.path.join(kale_marshal_dir, f))]
-
+    marshal.set_data_dir(kale_marshal_dir)
     return {os.path.splitext(f)[0]:
-            resource_load(os.path.join(kale_marshal_dir, f))
-            for f in load_file_names}
+            marshal.load(os.path.splitext(f)[0])
+            for f in os.listdir(kale_marshal_dir)}
 
 
 def explore_notebook(request, source_notebook_path):
@@ -185,9 +187,19 @@ def explore_notebook(request, source_notebook_path):
     step_name = os.getenv(KALE_PIPELINE_STEP_ENV, None)
     kale_marshal_dir = _get_kale_marshal_dir(source_notebook_path)
 
+    final_snapshot = str(os.getenv(KALE_SNAPSHOT_FINAL_ENV, "false")).lower()
+    if final_snapshot == "true":
+        final_snapshot = True
+    elif final_snapshot == "false":
+        final_snapshot = False
+    else:
+        raise ValueError("Env %s: Expected 'true' or 'false', but got: %s"
+                         % (KALE_SNAPSHOT_FINAL_ENV, final_snapshot))
+
     if step_name and os.path.exists(kale_marshal_dir):
         return {"is_exploration": True,
-                "step_name": step_name}
+                "step_name": step_name,
+                "final_snapshot": final_snapshot}
     return {"is_exploration": False,
             "step_name": ""}
 
@@ -214,3 +226,11 @@ def find_poddefault_labels_on_server(request):
     request.log.info("PodDefault labels applied on server: %s",
                      ", ".join(["%s: %s" % (k, v) for k, v in labels.items()]))
     return labels
+
+
+def get_namespace(request):
+    """Retrieve the namespace of the notebook."""
+    request.log.info("Retrieving notebook's namespace...")
+    namespace = podutils.get_namespace()
+    request.log.info("Notebook's namespace is '%s'", namespace)
+    return namespace
